@@ -1,6 +1,7 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
+// Copyright Dirk Lemstra 2013-2015
 //
 // Definition of Image, the representation of a single image in Magick++
 //
@@ -128,7 +129,7 @@ namespace Magick
 
     // Anti-alias Postscript and TrueType fonts (default true)
     void antiAlias(const bool flag_);
-    bool antiAlias(void);
+    bool antiAlias(void) const;
 
     // Time in 1/100ths of a second which must expire before
     // displaying the next image in an animated sequence.
@@ -201,7 +202,7 @@ namespace Magick
 
     // Colormap size (number of colormap entries)
     void colorMapSize(const size_t entries_);
-    size_t colorMapSize(void);
+    size_t colorMapSize(void) const;
 
     // Image Color Space
     void colorSpace(const ColorspaceType colorSpace_);
@@ -317,6 +318,10 @@ namespace Magick
     void iptcProfile(const Blob &iptcProfile_);
     Blob iptcProfile(void) const;
 
+    // Returns true if none of the pixels in the image have an alpha value
+    // other than OpaqueAlpha (QuantumRange).
+    bool isOpaque(void) const;
+
     // Does object contain valid image?
     void isValid(const bool isValid_);
     bool isValid(void) const;
@@ -413,6 +418,10 @@ namespace Magick
     void quantizeTreeDepth(const size_t treeDepth_);
     size_t quantizeTreeDepth(void) const;
 
+    // Suppress all warning messages. Error messages are still reported.
+    void quiet(const bool quiet_);
+    bool quiet(void) const;
+
     // The type of rendering intent
     void renderingIntent(const RenderingIntent renderingIntent_);
     RenderingIntent renderingIntent(void) const;
@@ -522,7 +531,7 @@ namespace Magick
     std::string tileName(void) const;
 
     // Number of colors in the image
-    size_t totalColors(void);
+    size_t totalColors(void) const;
 
     // Rotation to use when annotating with text or drawing
     void transformRotation(const double angle_);
@@ -638,11 +647,11 @@ namespace Magick
     void artifact(const std::string &name_,const std::string &value_);
 
     // Returns the value of the artifact with the specified name.
-    std::string artifact(const std::string &name_);
+    std::string artifact(const std::string &name_) const;
 
     // Access/Update a named image attribute
     void attribute(const std::string name_,const std::string value_);
-    std::string attribute(const std::string name_ );
+    std::string attribute(const std::string name_ ) const;
 
     // Extracts the 'mean' from the image and adjust the image to try
     // make set its gamma appropriatally.
@@ -1088,7 +1097,7 @@ namespace Magick
       const double hue_);
 
     // Returns the normalized moments of one or more image channels.
-    ImageMoments moments(void);
+    ImageMoments moments(void) const;
 
     // Applies a kernel to the image according to the given mophology method.
     void morphology(const MorphologyMethod method_,const std::string kernel_,
@@ -1259,6 +1268,9 @@ namespace Magick
     void reduceNoise(void);
     void reduceNoise(const double order_);
 
+    // Resets the image page canvas and position.
+    void repage();
+
     // Resize image in terms of its pixel size.
     void resample(const Geometry &geometry_);
 
@@ -1300,7 +1312,7 @@ namespace Magick
       const double sigma_,const double threshold_);
 
     // Separates a channel from the image and returns it as a grayscale image.
-    Image separate(const ChannelType channel_);
+    Image separate(const ChannelType channel_) const;
 
     // Applies a special effect to the image, similar to the effect achieved in
     // a photo darkroom by sepia toning.  Threshold ranges from 0 to 
@@ -1364,6 +1376,9 @@ namespace Magick
 
     // Splice the background color into the image.
     void splice(const Geometry &geometry_);
+    void splice(const Geometry &geometry_,const Color &backgroundColor_);
+    void splice(const Geometry &geometry_,const Color &backgroundColor_,
+      const GravityType gravity_);
 
     // Spread pixels randomly within image by specified ammount
     void spread(const size_t amount_=3);
@@ -1443,7 +1458,7 @@ namespace Magick
     void trim(void);
 
     // Returns the unique colors of an image.
-    Image uniqueColors(void);
+    Image uniqueColors(void) const;
 
     // Replace image with a sharpened version of the original image
     // using the unsharp mask algorithm.
@@ -1527,18 +1542,12 @@ namespace Magick
     // Prepare to update image (copy if reference > 1)
     void modifyImage(void);
 
-    // Register image with image registry or obtain registration id
-    ::ssize_t registerId(void);
-
     // Replace current image (reference counted)
     MagickCore::Image *replaceImage(MagickCore::Image *replacement_);
 
     // Test for ImageMagick error and throw exception if error
     void throwImageException(void) const;
 
-    // Unregister image from image registry
-    void unregisterId(void);
-
   private:
 
     void read(MagickCore::Image *image,
